"""
automatically invoked app factory
"""
import logging
import os

from flask import Flask
from flask_cors import CORS  # for debugging
# the currently available stubs for flask_migrate are old (they depend on sqlalchemy 1.4 types)
from flask_migrate import Migrate, upgrade  # type: ignore

from compendium_v2 import config, environment
from compendium_v2.db import db


def _create_app(app_config) -> Flask:
    # used by sphinx to create documentation without config and db migrations
    app = Flask(__name__)
    CORS(app)

    app.config['CONFIG_PARAMS'] = app_config

    from compendium_v2.routes import default
    app.register_blueprint(default.routes, url_prefix='/')

    from compendium_v2.routes import api
    app.register_blueprint(api.routes, url_prefix='/api')

    return app


def _create_app_with_db(app_config) -> Flask:
    # used by the tests and the publishers
    app = _create_app(app_config)
    app.config['SQLALCHEMY_DATABASE_URI'] = app.config['CONFIG_PARAMS']['SQLALCHEMY_DATABASE_URI']

    if 'SQLALCHEMY_BINDS' in app.config['CONFIG_PARAMS']:
        # for the publishers
        app.config['SQLALCHEMY_BINDS'] = app.config['CONFIG_PARAMS']['SQLALCHEMY_BINDS']

    db.init_app(app)
    return app


def create_app() -> Flask:
    """
    overrides default settings with those found
    in the file read from env var SETTINGS_FILENAME

    :return: a new flask app instance
    """

    assert 'SETTINGS_FILENAME' in os.environ, "environment variable 'SETTINGS_FILENAME' is required"

    with open(os.environ['SETTINGS_FILENAME']) as f:
        app_config = config.load(f)

    app = _create_app_with_db(app_config)

    Migrate(app, db, directory=os.path.join(os.path.dirname(__file__), 'migrations'))

    logging.info('Flask app initialized')

    environment.setup_logging()

    # run migrations on startup
    with app.app_context():
        upgrade()

    return app
