import React, {ReactElement, useEffect} from 'react';
import {Button, Dropdown, ButtonToolbar, Col} from 'react-bootstrap';
import Form from 'react-bootstrap/Form';
import {FilterSelection, Nren} from "../../Schema";

interface inputProps {
    filterOptions: { availableNrens: Nren[], availableYears: number[] }
    filterSelection: FilterSelection
    setFilterSelection: React.Dispatch<React.SetStateAction<FilterSelection>>
    max1year?: boolean
    coloredYears?: boolean
}

function Filter({ filterOptions, filterSelection, setFilterSelection, max1year = false, coloredYears = false }: inputProps): ReactElement {

    // State to track whether we can add a third column to the nren popup
    const [isThirdColumnVisible, setThirdColumnVisible] = React.useState(true);

    useEffect(() => {
        const handleWindowResize = () => setThirdColumnVisible(window.innerWidth >= 992);
        window.addEventListener('resize', handleWindowResize);
        return () => {
            window.removeEventListener('resize', handleWindowResize);
        };
    }, []);

    if (max1year && filterSelection.selectedYears.length > 1) {
        const maxYear = Math.max(...filterSelection.selectedYears);
        setFilterSelection({
            selectedYears: [maxYear],
            selectedNrens: [...filterSelection.selectedNrens]
        });
    }

    const handleNrenClick = (nren: string) => {
        if (filterSelection.selectedNrens.includes(nren)) {
            setFilterSelection({
                selectedYears: [...filterSelection.selectedYears],
                selectedNrens: filterSelection.selectedNrens.filter((e) => e !== nren)
            });
        } else {
            setFilterSelection({
                selectedYears: [...filterSelection.selectedYears],
                selectedNrens: [...filterSelection.selectedNrens, nren]
            });
        }
    };

    const handleYearClick = (year: number) => {
        if (filterSelection.selectedYears.includes(year)) {
            setFilterSelection({
                selectedYears: filterSelection.selectedYears.filter((e) => e !== year),
                selectedNrens: [...filterSelection.selectedNrens]
            });
        } else {
            setFilterSelection({
                selectedYears: max1year ? [year] : [...filterSelection.selectedYears, year],
                selectedNrens: [...filterSelection.selectedNrens]
            });
        }
    };

    const handleSelectAllNrensClick = () => {
        setFilterSelection({
            selectedYears: [...filterSelection.selectedYears],
            selectedNrens: filterOptions.availableNrens.map(n => n.name)
        });
    }

    const handleDeselectAllNrensClick = () => {
        setFilterSelection({
            selectedYears: [...filterSelection.selectedYears],
            selectedNrens: []
        });
    }

    // flex wrap in the column direction causes issues so we prepare the columns ourselves:
    const nrOfColumns = isThirdColumnVisible ? 3 : 2;
    const nrOfRows = Math.ceil(filterOptions.availableNrens.length / nrOfColumns);
    const grid: Nren[][] = Array.from(Array(nrOfColumns), () => []);
    filterOptions.availableNrens.sort().forEach((nren, idx) => {
        const col = Math.floor(idx / nrOfRows);
        grid[col].push(nren);
    });

    return (
        <>
            <Col xs={3}>
                <Dropdown autoClose="outside" className="m-3">
                    <Dropdown.Toggle id="nren-dropdown-toggle" variant="compendium">Select NRENs&nbsp;&nbsp;&nbsp;&nbsp;</Dropdown.Toggle>
                    <Dropdown.Menu style={{"borderRadius": 0}}>
                        <div className="d-flex fit-max-content mt-4 mx-3">
                            {grid.map((column, columnIndex) => (
                                <div key={columnIndex} className="flex-fill">
                                    {column.map((nren) => (
                                        <div className="filter-dropdown-item flex-fill py-1 px-3" key={nren.name} onClick={() => (handleNrenClick(nren.name))}>
                                            <Form.Check type="checkbox">
                                                <Form.Check.Input readOnly type="checkbox" checked={filterSelection.selectedNrens.includes(nren.name)} className="nren-checkbox"/>
                                                <Form.Check.Label className="nren-checkbox-label">{nren.name} <span style={{'fontWeight': 'lighter'}}>({nren.country})</span></Form.Check.Label>
                                            </Form.Check>
                                        </div>
                                    ))}
                                </div>
                            ))}
                        </div>
                        <div className="d-flex fit-max-content gap-2 mx-4 my-3">
                            <Button variant="compendium" className="flex-fill" onClick={handleSelectAllNrensClick}>Select all NRENs</Button>
                            <Button variant="compendium" className="flex-fill" onClick={handleDeselectAllNrensClick}>Unselect all NRENs</Button>
                        </div>
                    </Dropdown.Menu>
                </Dropdown>
            </Col>
            <Col>
                <ButtonToolbar className="d-flex justify-content-end gap-2 m-3">
                    { filterOptions.availableYears.sort().map(year => (
                        <Button
                                variant={ coloredYears ? "compendium-year-" + (year % 9) : "compendium-year"}
                                key={year}
                                active={filterSelection.selectedYears.includes(year)}
                                onClick={() => (handleYearClick(year))}>
                            {year}
                        </Button>
                    ))}
                </ButtonToolbar>
            </Col>
        </>

    );
}

export default Filter;
