import React, { useMemo, useState } from "react";
import { Table } from "react-bootstrap";
import { Chart as ChartJS, CategoryScale, LinearScale, BarElement, Title, Tooltip, Legend } from 'chart.js';

import { ChargingStructure, FilterSelection } from "../Schema";
import { createChargingStructureDataLookup, getYearsAndNrens, loadDataWithFilterSelectionFallback } from "../helpers/dataconversion";
import ColorPill from "../components/ColorPill";
import DataPage from "../components/DataPage";
import Filter from "../components/graphing/Filter";
import { Sections } from "../helpers/constants";


ChartJS.register(
    CategoryScale,
    LinearScale,
    BarElement,
    Title,
    Tooltip,
    Legend
);

interface inputProps {
    filterSelection: FilterSelection
    setFilterSelection: React.Dispatch<React.SetStateAction<FilterSelection>>
}

function ChargingStructurePage({ filterSelection, setFilterSelection }: inputProps): React.ReactElement {
    const [chargingStructureData, setChargingStructureData] = useState<ChargingStructure[]>([]);

    const { years, nrens } = useMemo(
        () => getYearsAndNrens(chargingStructureData),
        [chargingStructureData]
    );

    const selectedData = (chargingStructureData).filter(data =>
        filterSelection.selectedYears.includes(data.year) && filterSelection.selectedNrens.includes(data.nren)
    );

    const dataLookup = createChargingStructureDataLookup(selectedData);

    React.useEffect(() => {
        loadDataWithFilterSelectionFallback('/api/charging/', setChargingStructureData, setFilterSelection);
    }, [setFilterSelection]);

    const filterNode = <Filter
        filterOptions={{ availableYears: [...years], availableNrens: [...nrens.values()] }}
        filterSelection={filterSelection}
        setFilterSelection={setFilterSelection}
        coloredYears
    />

    return (
        <DataPage title="Charging Structure" category={Sections.Organisation} filter={filterNode}>
            <Table className="charging-struct-table" striped bordered responsive >
                <colgroup>
                    <col span={1} style={{ width: "10%" }} />
                    <col span={1} style={{ width: "18%" }} />
                    <col span={1} style={{ width: "18%" }} />
                    <col span={1} style={{ width: "18%" }} />
                    <col span={1} style={{ width: "18%" }} />
                    <col span={1} style={{ width: "18%" }} />
                </colgroup>
                <thead>
                    <tr >
                        <th></th>
                        <th>Flat fee based on bandwidth</th>
                        <th>Usage based fee</th>
                        <th>Combination flat fee & usage basedfee  </th>
                        <th>No Direct Charge</th>
                        <th>Other</th>
                    </tr>
                </thead>
                <tbody>
                    {Array.from(dataLookup.entries()).map(([nren, nrenMap]) => (
                            <tr key={nren}>
                                <td>{nren}</td>
                                {["flat_fee", "usage_based_fee", "combination", "no_charge", "other"].map(discriminator => (
                                    <td key={discriminator}>
                                        {Array.from(nrenMap.entries()).map(([year, chargingMechanism]) => (
                                            <ColorPill key={year} year={year} active={chargingMechanism == discriminator} />
                                        ))}
                                    </td>
                                ))}
                            </tr>
                        ))}
                </tbody>

            </Table>
        </DataPage>
    );
}
export default ChargingStructurePage;
