import React, { useEffect, useMemo, useState } from 'react';
import { Bar } from 'react-chartjs-2';
import { Col, Container, Row, Table } from "react-bootstrap";
import { Chart as ChartJS, CategoryScale, LinearScale, BarElement, Title, Tooltip, Legend, Chart } from 'chart.js';

import { FundingSource, FilterSelection } from "../Schema";
import { createFundingSourceDataset, getYearsAndNrens, loadDataWithFilterSelectionFallback } from "../helpers/dataconversion";
import DataPage from '../components/DataPage';
import Filter from "../components/graphing/Filter"
import { Sections } from '../helpers/constants';
import ColorPill from '../components/ColorPill';
import ColorBadge from '../components/ColorBadge';

ChartJS.defaults.font.size = 16;


ChartJS.register(
    CategoryScale,
    LinearScale,
    BarElement,
    Title,
    Tooltip,
    Legend
);


export const chartOptions = {
    maintainAspectRatio: false,
    animation: {
        duration: 0
    },
    plugins: {
        legend: {
            display: false,
            labels: {
                boxWidth: 20,
                boxHeight: 30,
                pointStyle: "rectRounded",
                borderRadius: 6,
                useBorderRadius: true,

                },
            },
        },
    scales: {
        x: {
            position: 'top' as const,
            stacked: true,
            ticks: {
                font:{
                size: 20,
                family: 'Open Sans, sans-serif',
                weight: 'bold',
                },
                callback: (value: string | number) => {
                    if (typeof value === 'number') {
                        return `${value}%`;
                    }
                    return value;
                },
            },

        },
        x2: {
            stacked: true,
            ticks: {
                callback: (value: string | number) => {
                    if (typeof value === 'number') {
                        return `${value}%`;
                    }
                    return value;
                },
                font: {
                    size: 20,
                    family: 'Open Sans, sans-serif',
                    weight: 'bold',
                }
            },
            grid: {
                display: false,
                drawTicks: false,
                drawBorder: false,
            },
            
            min : 0,
            max : 100
        },
        y: {
            stacked: true,
            ticks: {
                autoSkip: false,
                font: {
                    size: 20,
                    family: 'Open Sans, sans-serif',
                    weight: 'bold',
                }
            },
        },
    },
    indexAxis: "y" as const
};

interface inputProps {
    filterSelection: FilterSelection
    setFilterSelection: React.Dispatch<React.SetStateAction<FilterSelection>>
}

function FundingSourcePage({ filterSelection, setFilterSelection }: inputProps) {
    const [fundingSourceData, setFundingSourceData] = useState<FundingSource[]>([]);

    const { years, nrens } = useMemo(
        () => getYearsAndNrens(fundingSourceData),
        [fundingSourceData]
    );
    const fundingSourceDataset = createFundingSourceDataset(fundingSourceData);

    fundingSourceDataset.datasets.forEach(dataset => {
        dataset.hidden = !filterSelection.selectedYears.includes(parseInt(dataset.stack));
    });

    // remove the datapoints and labels for the nrens that aren't selected
    // unfortunately we cannot just hide them because graph.js doesn't want
    // to create a stack from a single dataset
    fundingSourceDataset.datasets.forEach(dataset => {
        dataset.data = dataset.data.filter((e, i) => {
            return filterSelection.selectedNrens.includes(fundingSourceDataset.labels[i]);
        });
    });
    fundingSourceDataset.labels = fundingSourceDataset.labels.filter((e) => filterSelection.selectedNrens.includes(e));

    useEffect(() => {
        loadDataWithFilterSelectionFallback('/api/funding/', setFundingSourceData, setFilterSelection);
    }, [setFilterSelection]);

    const filterNode = <Filter
        filterOptions={{ availableYears: [...years], availableNrens: [...nrens.values()] }}
        filterSelection={filterSelection}
        setFilterSelection={setFilterSelection}
    />

    const numNrens = filterSelection.selectedNrens.length;
    const numYears = filterSelection.selectedYears.length;
    let heightPerBar = 4; // every added bar should give this much additional height

    // set a minimum height of 20rem, additional years need some more space
    const height = Math.max(numNrens * heightPerBar + (numYears * 2 * heightPerBar), 20);
    return (
        <DataPage title="Funding Source" category={Sections.Organisation} filter={filterNode}>
            <div>
        <Container  className=" border rounded-3 border-1 funding-chart-Legend" 
        style={{ marginTop: '20px', marginBottom: '20px', padding: '10px'}}>
            <Row fluid className="justify-content-center"> 
                <Col className="d-flex align-items-center">
                <ColorBadge key={0} index={0}/>
                <span className='bold-caps-16pt'>CLIENT INSTITUTIONS</span></Col>
                <Col className="d-flex align-items-center" >
                <ColorBadge key={1} index={1}/>
                <span className='bold-caps-16pt'>COMMERCIAL</span></Col>
                <Col className="d-flex align-items-center" >
                <ColorBadge key={2} index={2}/>
                <span className='bold-caps-16pt'>EUROPEAN FUNDING</span></Col>
                <Col className="d-flex align-items-center" >
                <ColorBadge key={3} index={3}/>
                <span className='bold-caps-16pt'>GOV/PUBLIC BODIES</span></Col>
                <Col className="d-flex align-items-center" >
                <ColorBadge key={4} index={4}/>
                <span className='bold-caps-16pt dark-teal'>OTHER</span></Col>
            </Row>
        </Container>  

            <div className="chart-container" style={{ 'height': `${height}rem` }}>
                <Bar
                    data={fundingSourceDataset}
                    options={chartOptions}
                />
            </div>
            </div>
        </DataPage>
    );
}
export default FundingSourcePage;
