import React, { useEffect, useMemo, useState } from 'react';
import { Bar } from 'react-chartjs-2';
import { Chart as ChartJS, CategoryScale, LinearScale, BarElement, Title, Tooltip, Legend } from 'chart.js';

import { NrenStaff, FilterSelection } from "../Schema";
import { createNRENStaffDataset, getYearsAndNrens, loadDataWithFilterSelectionFallback } from "../helpers/dataconversion";
import DataPage from '../components/DataPage';
import Filter from "../components/graphing/Filter"
import { Sections } from '../helpers/constants';

ChartJS.defaults.font.size = 14;

ChartJS.register(
    CategoryScale,
    LinearScale,
    BarElement,
    Title,
    Tooltip,
    Legend
);

const chartOptions = {
    maintainAspectRatio: false,
    animation: {
        duration: 0
    },
    plugins: {
        legend: {
            display: true,
            position: 'bottom' as const
        },
        tooltip: {
            callbacks: {
                label: function (tooltipItem) {
                    let label = tooltipItem.dataset.label || '';

                    if (tooltipItem.parsed.x !== null) {
                        label += `: ${tooltipItem.parsed.x}%`
                    }
                    return label;
                }
            },

        },
    },

    scales: {
        x: {
            stacked: true,
            ticks: {
                callback: (value: string | number, index: number) => {
                    return `${index * 10}%`;
                },
            },
        },
        y: {
            stacked: true,
            ticks: {
                autoSkip: false,
            },
        },
    },
    indexAxis: "y" as const
};

interface inputProps {
    filterSelection: FilterSelection
    setFilterSelection: React.Dispatch<React.SetStateAction<FilterSelection>>
    roles?: boolean
}

function StaffGraph({ filterSelection, setFilterSelection, roles = false }: inputProps) {
    const [staffData, setStaffData] = useState<NrenStaff[]>([]);

    const { years, nrens } = useMemo(
        () => getYearsAndNrens(staffData),
        [staffData]
    );
    const nrenStaffDataset = createNRENStaffDataset(staffData, roles, filterSelection.selectedYears[0]);

    nrenStaffDataset.datasets.forEach(dataset => {
        dataset.hidden = !filterSelection.selectedYears.includes(parseInt(dataset.stack));
    });

    // remove the datapoints and labels for the nrens that aren't selected
    // unfortunately we cannot just hide them because graph.js doesn't want
    // to create a stack from a single dataset
    nrenStaffDataset.datasets.forEach(dataset => {
        dataset.data = dataset.data.filter((e, i) => {
            return filterSelection.selectedNrens.includes(nrenStaffDataset.labels[i]);
        });
    });
    nrenStaffDataset.labels = nrenStaffDataset.labels.filter((e) => filterSelection.selectedNrens.includes(e));

    useEffect(() => {
        loadDataWithFilterSelectionFallback('/api/staff/', setStaffData, setFilterSelection);
    }, [setFilterSelection]);

    const filterNode = <Filter
        max1year
        filterOptions={{ availableYears: [...years], availableNrens: [...nrens.values()] }}
        filterSelection={filterSelection}
        setFilterSelection={setFilterSelection}
    />

    const numNrens = filterSelection.selectedNrens.length;
    let heightPerBar = 1.5; // every added bar should give this much additional height

    // set a minimum height of 20rem
    const height = Math.max(numNrens * heightPerBar, 20);

    const text = roles ? "Roles" : "Employment";
    return (
        <DataPage title={"NREN Staff " + text} category={Sections.Organisation} filter={filterNode}>
            <div className="chart-container" style={{ 'height': `${height}rem` }}>
                <Bar
                    data={nrenStaffDataset}
                    options={chartOptions}
                />
            </div>
        </DataPage>
    );
}
export default StaffGraph;
